@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -20,6 +20,7 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 
+
 #include <libkern/c++/OSContainers.h>
 #include <IOKit/system.h>
 #include <IOKit/IOLib.h>
@@ -35,10 +36,15 @@
 #include <stddef.h>
 #endif
 
+#if IOGRAPHICSTYPES_REV >= 8
+#define TIGER 1
+#endif
+
 #include "IVAD.h"
 
 enum {
 	kIVADInvertDataMask	= 0x100,
+	kIVADUsePWMAddress	= 0x200
 };
 
 static IOReturn
@@ -435,6 +441,10 @@ bool AppleIVAD::start( IOService* provider )
 		return( false );
 
 	fDisplayParams = OSDynamicCast( OSDictionary, getProperty( gIODisplayParametersKey ) );
+#if TIGER
+	if (fDisplayParams)
+	    fDisplayParams = (OSDictionary *) fDisplayParams->copyCollection();
+#endif
 	if ( !fDisplayParams )
 		return( false );
 
@@ -1044,6 +1054,10 @@ bool AppleIVAD2::start( IOService* provider )
     fIVAD = provider;
 
 	fDisplayParams = OSDynamicCast( OSDictionary, getProperty( gIODisplayParametersKey ) );
+#if TIGER
+	if (fDisplayParams)
+	    fDisplayParams = (OSDictionary *) fDisplayParams->copyCollection();
+#endif
 	if ( !fDisplayParams )
 		return( false );
 
@@ -1948,6 +1962,10 @@ bool AppleVCP::start( IOService* provider )
 		return( false );
 
 	fDisplayParams = OSDynamicCast( OSDictionary, getProperty( gIODisplayParametersKey ) );
+#if TIGER
+	if (fDisplayParams)
+	    fDisplayParams = (OSDictionary *) fDisplayParams->copyCollection();
+#endif
 	if ( !fDisplayParams )
 		return( false );
 
@@ -2291,6 +2309,10 @@ bool ApplePanel::start( IOService * provider )
         return( false );
 
     fDisplayParams = OSDynamicCast(OSDictionary, getProperty(gIODisplayParametersKey));
+#if TIGER
+    if (fDisplayParams)
+	fDisplayParams = (OSDictionary *) fDisplayParams->copyCollection();
+#endif
     if( !fDisplayParams)
         return( false );
 
@@ -2352,8 +2374,6 @@ bool ApplePanel::setDisplay( IODisplay * display )
 
     } while( false );
 
-    display->setProperty( gIODisplayParametersKey, fDisplayParams);
-
     num = OSNumber::withNumber( kAppleOnboardGUID, 64 );
     display->setProperty( gIODisplayGUIDKey, num);
     num->release();
@@ -2400,7 +2420,7 @@ bool ApplePanelA::start( IOService * provider )
 
     if( !super::start( provider))
         return( false );
-    
+
    // Assume that there is no NVRAM
     fNoNVRAM  = true;
     if ((regEntry = IORegistryEntry::fromPath("/options", gIODTPlane)))
@@ -2428,7 +2448,7 @@ bool ApplePanelA::setDisplay( IODisplay * display )
 
         fFramebuffer = fb;
 
-        IODisplay::addParameter( fDisplayParams, gIODisplayBrightnessKey,
+	IODisplay::addParameter( fDisplayParams, gIODisplayBrightnessKey,
 //                                  fPanelInfo->min, fPanelInfo->max );
 //	                            0 + (16*fPanelInfo->min - fPanelInfo->max) / 15, fPanelInfo->max );
                                     0, kApplePanelLevels );
@@ -2458,6 +2478,8 @@ bool ApplePanelA::setDisplay( IODisplay * display )
             fFramebuffer->doControl( kPanelABacklightControl, &info);
         }
 
+	display->setProperty( gIODisplayParametersKey, fDisplayParams );
+
         return( true );
     } else
         return( false );
@@ -2609,6 +2631,8 @@ bool ApplePMUPanel::setDisplay( IODisplay * display )
                                      kSTNContrastMin, kSTNContrastMax );
         }
     } while( false );
+
+    display->setProperty( gIODisplayParametersKey, fDisplayParams );
     
     return( true );
 }
